/*
	Directory server components by Catid@pacbell.net
	------------------------------------------------
	port 4990 - directory server list upload
	port 4991 - zone update
	port NULL - directory server list download
	port NULL - zone pings
*/

#ifndef DSERV_H
#define DSERV_H

#include "host.h"
#include "datatypes.h"
#include "sockets.h"

// Delay between consecutive ping requests. 200 milliseconds
#define SWEEP_TIME          200

// Delay between consecutive zone list requests. 1 minute
#define DOWNLOAD_TIME     60000

// Number of pings dropped before the host is disconnected. 5 pings
#define DROP_LIMIT            5

// Limit to time between direct updates (from subgame) before the password is reclaimed. 1 day
#define RECLAIM_DELAY  86400000

bool nameMatch(const char * a, const char * b);
bool passMatch(const char * a, const char * b);
bool validName(const char * name);
bool validUpdate(char * msg, Uint32 len);

struct passwordEntry
{
	char name[32];
	char pass[16];

	Uint32 lastTime;

	passwordEntry(char * nname, char * npass);
	bool validate(char * tpass);
	bool timeout();
};

class passwordList
{
	_linkedlist <passwordEntry> plist;

public:
	bool validate(char * name, char * pass);
};

struct serverEntry
{
	Uint32 ip;
	Uint16 port;
	Uint16 population;
	Uint16 scoring;		// Connected to a biller?
	Uint32 version;
	char name[32];		// NUL-terminated zone name
	char comment[256];	// NUL-terminated zone comment

	serverEntry();
	serverEntry(Uint32 nip, Uint16 nport, Uint16 npopulation, Uint16 nscoring, Uint32 nversion, char * nname, char * ncomment);
	bool validate(Uint32 tip, Uint16 tport);
	void set(char * nname, char * ncomment);

	// Ping sweep
	bool pinging;		// Kicking ass and taking names
	Uint32 highPing;
	Uint32 dropped;		// Total pings declared MIA
	Uint32 lastTime;	// Time since last ping

	void startPinging();
	void gotPong(Uint32 pingTime, Uint32 npopulation);
	bool resendPing();
	bool timeout();
};

class zoneList
{
	_listnode <serverEntry> * target;

	void addZone(serverEntry * se);

public:
	_linkedlist <serverEntry> zlist;

	zoneList();

	serverEntry * find(Uint32 ip, Uint16 port);
	serverEntry * getNext();

	void killZone(serverEntry * se);
	void updateZone(Uint32 ip, Uint16 port, Uint16 population, Uint16 scoring, Uint32 version, char * name, char * comment);
};

class directoryServer
{
	passwordList plist;

	hostList download;	// (client) Port on which we download the zone list
	hostList upload;	// (server) Port on which we upload the zone list
	UDPSocket update;	// (server) Port on which we accept zone status directly
	UDPSocket sweep;	// (client) Port on which we ping-sweep the zone list

	Uint32 lastTime;	// Last time we started pinging a host
	Uint32 iteration;	// Last iteration ticks

	bool busy;			// When a hostList is active, don't do ping-sweeps
	Uint32 lastDownload;// Last time we finished downloading the server list
	Uint32 state;		// Selected directory server

	void sendPing(Uint32 ip, Uint16 port);
	void sendPong(Uint32 ip, Uint16 port, Uint32 pingTime);
	void resetPings();

public:
	zoneList zlist;

	directoryServer();
	~directoryServer();

	clusterMessage * makeZoneList(Uint32 min);

	void doEvents();
};

#endif	// DSERV_H
